@@ -19,6 +19,7 @@
 #define _XOPEN_SOURCE
 #define _GNU_SOURCE
 
+#include <limits.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -366,7 +367,7 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
         g_error_free (err);
         goto error;
     }
-        
+
     contents = g_mapped_file_get_contents (fp);
     file_len = g_mapped_file_get_length (fp);
     end = contents + file_len;
@@ -399,6 +400,12 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
             if (value) {
                 /* Space for the leading newline too. */
                 value_pos = value_p - value;
+                if (INT_MAX - (1 + value_length + 1) < value_pos) {
+                    g_set_error (error,
+                                 g_quark_from_static_string ("whoopsie-quark"),
+                                 0, "Report value too long.");
+                    goto error;
+                }
                 value = g_realloc (value, value_pos + 1 + value_length + 1);
                 value_p = value + value_pos;
                 *value_p = '\n';
@@ -707,9 +714,9 @@ process_existing_files (const char* report_dir)
             continue;
         } else if (online_state && parse_and_upload_report (crash_file)) {
             if (!mark_handled (crash_file))
-                log_msg ("Unable to mark report as seen (%s)\n", crash_file);
-
-        } 
+                log_msg ("Unable to mark report as seen (%s) removing it.\n", crash_file);
+                g_unlink (crash_file);
+        }
 
         g_free (upload_file);
         g_free (crash_file);
@@ -843,7 +850,9 @@ drop_privileges (GError** error)
 void
 network_changed (gboolean available)
 {
-    log_msg (available ? "online\n" : "offline\n");
+    if (online_state != available)
+        log_msg (available ? "online\n" : "offline\n");
+
     if (!available) {
         online_state = FALSE;
         return;
@@ -986,6 +995,7 @@ main (int argc, char** argv)
 
     /* Publish whoopsie-id */
     g_file_set_contents (WHOOPSIE_ID_PATH, whoopsie_identifier, -1, NULL);
+    chmod (WHOOPSIE_ID_PATH, 00600);
 
     create_crash_directory ();
 
